@@ -1,18 +1,19 @@
-// Copyright 2013-2015, The Gtk-rs Project Developers.
-// See the COPYRIGHT file at the top-level directory of this distribution.
-// Licensed under the MIT license, see the LICENSE file or <http://opensource.org/licenses/MIT>
+// Take a look at the license at the top of the repository in the LICENSE file.
 
+mod font_extents;
 mod font_face;
 mod font_options;
+mod glyph;
 mod scaled_font;
+mod text_cluster;
+mod text_extents;
+mod user_fonts;
 
-pub use enums::{
+pub use crate::enums::{
     Antialias, FontSlant, FontType, FontWeight, HintMetrics, HintStyle, SubpixelOrder,
     TextClusterFlags,
 };
 
-pub use ffi::{FontExtents, Glyph, TextCluster, TextExtents};
-
 /* TODO
  Allocates an array of cairo_glyph_t's. This function is only useful in
  implementations of cairo_user_scaled_font_text_to_glyphs_func_t where the user
@@ -39,6 +40,11 @@ impl TextCluster {
 }
 */
 
+pub use self::font_extents::FontExtents;
 pub use self::font_face::FontFace;
 pub use self::font_options::FontOptions;
+pub use self::glyph::Glyph;
 pub use self::scaled_font::ScaledFont;
+pub use self::text_cluster::TextCluster;
+pub use self::text_extents::TextExtents;
+pub use self::user_fonts::UserFontFace;
